@@ -13,6 +13,8 @@
 #include <string.h>
 #include <stdlib.h>     /* strtol */
 
+array plugin_stats; /* global */
+
 /**
  * like all glue code this file contains functions which
  * are the external interface of lighttpd. The functions
@@ -235,7 +237,7 @@ int config_plugin_values_init_block(server * const srv, const array * const ca,
                     char *e;
                     long l = strtol(v, &e, 10);
                     if (e != v && !*e && l >= 0) {
-                        cpv->v.shrt = (unsigned int)l;
+                        cpv->v.u = (unsigned int)l;
                         break;
                     }
                 }
@@ -305,8 +307,9 @@ int config_plugin_values_init(server * const srv, void *p_d, const config_plugin
     /* traverse config contexts twice: once to count, once to store matches */
 
     for (uint32_t u = 0; u < srv->config_context->used; ++u) {
-        const array *ca =
-          ((data_config const *)srv->config_context->data[u])->value;
+        const data_config * const dc =
+          (const data_config *)srv->config_context->data[u];
+        const array * const ca = dc->value;
 
         matches[n] = 0;
         for (int i = 0; cpk[i].ktype != T_CONFIG_UNSET; ++i) {
@@ -318,12 +321,19 @@ int config_plugin_values_init(server * const srv, void *p_d, const config_plugin
 
             array_set_key_value(touched,cpk[i].k,cpk[i].klen,CONST_STR_LEN(""));
 
-            if (cpk[i].scope == T_CONFIG_SCOPE_SERVER && 0 != u) {
+            if (cpk[i].scope == T_CONFIG_SCOPE_CONNECTION || 0 == u) continue;
+
+            if (cpk[i].scope == T_CONFIG_SCOPE_SERVER)
                 /* server scope options should be set only in server scope */
                 log_error(srv->errh, __FILE__, __LINE__,
                   "DEPRECATED: do not set server options in conditionals, "
                   "variable: %s", cpk[i].k);
-            }
+            if (cpk[i].scope == T_CONFIG_SCOPE_SOCKET
+                && (dc->comp!=COMP_SERVER_SOCKET || dc->cond!=CONFIG_COND_EQ))
+                /* socket options should be set in socket or global scope */
+                log_error(srv->errh, __FILE__, __LINE__,
+                  "WARNING: %s must be in global scope or $SERVER[\"socket\"] "
+                  "with '==', or else is ignored", cpk[i].k);
         }
         if (matches[n]) contexts[n++] = (unsigned short)u;
     }
@@ -559,6 +569,18 @@ static cond_result_t config_check_cond_nocache_eval(request_st * const r, const
 		match = (dc->cond == CONFIG_COND_MATCH);
 		match ^= (config_pcre_match(r, dc, l) > 0);
 		break;
+	case CONFIG_COND_PREFIX:
+	case CONFIG_COND_SUFFIX:
+		{
+			uint_fast32_t llen = buffer_clen(l);
+			uint_fast32_t dlen = buffer_clen(&dc->string);
+			uint_fast32_t off  = (dc->cond == CONFIG_COND_PREFIX)
+			                   ? 0
+			                   : llen - dlen; /*(underflow caught below)*/
+			match = !(dlen <= llen
+			          && 0 == memcmp(l->ptr+off, dc->string.ptr, dlen));
+		}
+		break;
 	default:
 		match = 1; /* return (cache->local_result = COND_RESULT_FALSE); below */
 		break;
