@@ -57,7 +57,7 @@ static void data_config_free(data_unset *d) {
 }
 
 data_config *data_config_init(void) {
-	static const struct data_methods fn = {
+	static const struct data_methods config_fn = {
 		data_config_copy,
 		data_config_free,
 		NULL
@@ -72,7 +72,7 @@ data_config *data_config_init(void) {
 	vector_config_weak_init(&ds->children);
 
 	ds->type = TYPE_CONFIG;
-	ds->fn = &fn;
+	ds->fn = &config_fn;
 
 	return ds;
 }
@@ -99,7 +99,7 @@ int data_config_pcre_compile(data_config * const dc, const int pcre_jit, log_err
 
     if (pcre_jit) {
         errcode = pcre2_jit_compile(dc->code, PCRE2_JIT_COMPLETE);
-        if (0 != errcode) {
+        if (0 != errcode && errcode != PCRE2_ERROR_JIT_BADOPTION) {
             pcre2_get_error_message(errcode, errbuf, sizeof(errbuf));
             log_error(errh, __FILE__, __LINE__,
                       "pcre2_jit_compile: %s, regex: %s",
