@@ -91,7 +91,7 @@ int http_response_buffer_append_authority(request_st * const r, buffer * const o
 			}
 			if (0 == listen_port) listen_port = r->con->srv->srvconf.port;
 			if (default_port != listen_port) {
-				buffer_append_string_len(o, CONST_STR_LEN(":"));
+				buffer_append_char(o, ':');
 				buffer_append_int(o, listen_port);
 			}
 		}
@@ -124,7 +124,7 @@ int http_response_redirect_to_directory(request_st * const r, int status) {
 	buffer_copy_buffer(vb, o);
 	buffer_append_string_encoded(vb, BUF_PTR_LEN(&r->uri.path),
 	                             ENCODING_REL_URI);
-	buffer_append_string_len(vb, CONST_STR_LEN("/"));
+	buffer_append_char(vb, '/');
 	if (!buffer_is_blank(&r->uri.query))
 		buffer_append_str2(vb, CONST_STR_LEN("?"),
 		                       BUF_PTR_LEN(&r->uri.query));
@@ -211,7 +211,7 @@ int http_response_handle_cachable(request_st * const r, const buffer *lmod, cons
 		/*(weak etag comparison must not be used for ranged requests)*/
 		int range_request = (0 != light_btst(r->rqst_htags, HTTP_HEADER_RANGE));
 		if (http_etag_matches(etag, vb->ptr, !range_request)) {
-			if (http_method_get_or_head(r->http_method)) {
+			if (http_method_get_head_query(r->http_method)) {
 				r->http_status = 304;
 				return HANDLER_FINISHED;
 			} else {
@@ -220,7 +220,7 @@ int http_response_handle_cachable(request_st * const r, const buffer *lmod, cons
 				return HANDLER_FINISHED;
 			}
 		}
-	} else if (http_method_get_or_head(r->http_method)
+	} else if (http_method_get_head_query(r->http_method)
 		   && (vb = http_header_request_get(r, HTTP_HEADER_IF_MODIFIED_SINCE,
 		                                    CONST_STR_LEN("If-Modified-Since")))
 		   && (lmod
@@ -951,7 +951,8 @@ static int http_response_process_headers(request_st * const restrict r, http_res
             /*(flag only for mod_proxy and mod_cgi (for now))*/
             if (opts->backend != BACKEND_PROXY && opts->backend != BACKEND_CGI)
                 continue;
-            if (r->http_version >= HTTP_VERSION_2) continue;
+            if (r->http_version >= HTTP_VERSION_2 && !r->h2_connect_ext)
+                continue;
             break;
           case HTTP_HEADER_CONNECTION:
             if (opts->backend == BACKEND_PROXY) continue;
@@ -961,6 +962,16 @@ static int http_response_process_headers(request_st * const restrict r, http_res
                 r->keep_alive = 0;
             if (r->http_version >= HTTP_VERSION_2) continue;
             break;
+          case HTTP_HEADER_CONTENT_TYPE:
+            if (end - value >= 22   /*(prefix match probably good enough)*/
+                && 0 == memcmp(value, "application/javascript", 22)) {
+                /* value = "text/javascript"; *//*(loses ";charset=...")*/
+                /* *(const char **)&end = value+sizeof("text/javascript")-1; */
+                /*(convert "application/javascript" to "text/javascript")*/
+                value += 7; /*(step over "applica", leaving "tion")*/
+                memcpy(s+(value-s)+1, "ext", 3);
+            }
+            break;
           case HTTP_HEADER_CONTENT_LENGTH:
             if (*value == '+') ++value;
             if (!r->resp_decode_chunked
