@@ -87,6 +87,7 @@ typedef struct {
 	buffer *cgi_handler;      /* dumb pointer */
 	http_response_opts opts;
 	plugin_config conf;
+	off_t orig_reqbody_length;
 } handler_ctx;
 
 typedef struct cgi_pid_t {
@@ -522,6 +523,13 @@ static handler_t cgi_response_headers(request_st * const r, struct http_response
     if (light_btst(r->resp_htags, HTTP_HEADER_UPGRADE)) {
         if (hctx->conf.upgrade && r->http_status == 101) {
             /* 101 Switching Protocols; transition to transparent proxy */
+            if (r->h2_connect_ext) {
+                r->http_status = 200; /* OK (response status for CONNECT) */
+                http_header_response_unset(r, HTTP_HEADER_UPGRADE,
+                                           CONST_STR_LEN("Upgrade"));
+                http_header_response_unset(r, HTTP_HEADER_OTHER,
+                                         CONST_STR_LEN("Sec-WebSocket-Accept"));
+            }
             http_response_upgrade_read_body_unknown(r);
         }
         else {
@@ -535,11 +543,20 @@ static handler_t cgi_response_headers(request_st * const r, struct http_response
           #endif
         }
     }
+    else if (__builtin_expect( (r->h2_connect_ext != 0), 0)
+             && r->http_status < 300) {
+        /*(not handling other 1xx intermediate responses here; not expected)*/
+        http_response_body_clear(r, 0);
+        r->handler_module = NULL;
+        r->http_status = 405; /* Method Not Allowed */
+        return HANDLER_FINISHED;
+    }
 
     if (hctx->conf.upgrade
         && !light_btst(r->resp_htags, HTTP_HEADER_UPGRADE)) {
         chunkqueue *cq = &r->reqbody_queue;
         hctx->conf.upgrade = 0;
+        r->reqbody_length = hctx->orig_reqbody_length;
         if (cq->bytes_out == (off_t)r->reqbody_length) {
             cgi_connection_close_fdtocgi(hctx); /*(closes hctx->fdtocgi)*/
         }
@@ -555,7 +572,7 @@ static handler_t cgi_local_redir(request_st * const r, handler_ctx * const hctx)
     buffer_clear(hctx->response);
     chunk_buffer_yield(hctx->response);
     http_response_reset(r); /*(includes r->http_status = 0)*/
-    plugins_call_handle_request_reset(r);
+    r->con->srv->plugins_request_reset(r);
     /*cgi_connection_close(hctx);*//*(already cleaned up and hctx is now invalid)*/
     return HANDLER_COMEBACK;
 }
@@ -622,6 +639,7 @@ static handler_t cgi_process_rd_revents(handler_ctx * const hctx, request_st * c
 			if (0 == r->http_status) r->http_status = 200; /* OK */
 		}
 		cgi_connection_close(hctx);
+		return HANDLER_FINISHED;
 	} else if (revents & FDEVENT_ERR) {
 		/* kill all connections to the cgi process */
 		cgi_connection_close(hctx);
@@ -768,7 +786,8 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 		}
 	}
 	else if (!(r->conf.stream_request_body /*(if not streaming request body)*/
-	           & (FDEVENT_STREAM_REQUEST|FDEVENT_STREAM_REQUEST_BUFMIN))) {
+	           & (FDEVENT_STREAM_REQUEST|FDEVENT_STREAM_REQUEST_BUFMIN))
+	         && !hctx->conf.upgrade) {
 		chunkqueue * const cq = &r->reqbody_queue;
 		chunk * const c = cq->first;
 		if (c && c == cq->last && c->type == FILE_CHUNK && c->file.is_temp) {
@@ -792,7 +811,7 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 
 	unsigned int bufsz_hint = 16384;
   #ifdef _WIN32
-	if (r->reqbody_length <= 1048576)
+	if (r->reqbody_length <= 1048576 && r->reqbody_length > 0)
 		bufsz_hint = (unsigned int)r->reqbody_length;
   #endif
 	if (-1 == to_cgi_fds[0] && fdevent_pipe_cloexec(to_cgi_fds, bufsz_hint)) {
@@ -825,8 +844,42 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 
 		/* create environment */
 
+		if (r->h2_connect_ext) {
+			/*(SERVER_PROTOCOL=HTTP/1.1 instead of HTTP/2.0)*/
+			r->http_version = HTTP_VERSION_1_1;
+			r->http_method = HTTP_METHOD_GET;
+		}
+		if (hctx->conf.upgrade) {
+			r->reqbody_length = hctx->orig_reqbody_length;
+			if (r->reqbody_length < 0)
+				r->reqbody_length = 0;
+		}
+
 		http_cgi_headers(r, &opts, cgi_env_add, env);
 
+		if (hctx->conf.upgrade)
+			r->reqbody_length = -1;
+		if (r->h2_connect_ext) {
+			r->http_version = HTTP_VERSION_2;
+			r->http_method = HTTP_METHOD_CONNECT;
+			/* https://datatracker.ietf.org/doc/html/rfc6455#section-4.1
+			 * 7. The request MUST include a header field with the name
+			 *    |Sec-WebSocket-Key|.  The value of this header field MUST be a
+			 *    nonce consisting of a randomly selected 16-byte value that has
+			 *    been base64-encoded (see Section 4 of [RFC4648]).  The nonce
+			 *    MUST be selected randomly for each connection.
+			 * Note: Sec-WebSocket-Key is not used in RFC8441;
+			 *       include Sec-WebSocket-Key for HTTP/1.1 compatibility;
+			 *       !!not random!! base64-encoded "0000000000000000" */
+			if (!http_header_request_get(r, HTTP_HEADER_OTHER,
+			                             CONST_STR_LEN("Sec-WebSocket-Key")))
+				cgi_env_add(env, CONST_STR_LEN("HTTP_SEC_WEBSOCKET_KEY"),
+				                 CONST_STR_LEN("MDAwMDAwMDAwMDAwMDAwMAo="));
+			/*(Upgrade and Connection should not exist for HTTP/2 request)*/
+			cgi_env_add(env, CONST_STR_LEN("HTTP_UPGRADE"), CONST_STR_LEN("websocket"));
+			cgi_env_add(env, CONST_STR_LEN("HTTP_CONNECTION"), CONST_STR_LEN("upgrade"));
+		}
+
 		/* for valgrind */
 		if (p->env.ld_preload) {
 			cgi_env_add(env, CONST_STR_LEN("LD_PRELOAD"), BUF_PTR_LEN(p->env.ld_preload));
@@ -965,12 +1018,18 @@ URIHANDLER_FUNC(cgi_is_handled) {
 	if (!S_ISREG(st->st_mode)) return HANDLER_GO_ON;
 	if (p->conf.execute_x_only == 1 && (st->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) == 0) return HANDLER_GO_ON;
 
+	if (__builtin_expect( (r->h2_connect_ext != 0), 0) && !p->conf.upgrade) {
+		r->http_status = 405; /* Method Not Allowed */
+		return HANDLER_FINISHED;
+	}
+
 	if (r->reqbody_length
 	    && p->tempfile_accum
 	    && !(r->conf.stream_request_body /*(if not streaming request body)*/
 	         & (FDEVENT_STREAM_REQUEST|FDEVENT_STREAM_REQUEST_BUFMIN))) {
 		/* store request body in single tempfile if not streaming request body*/
-		r->reqbody_queue.upload_temp_file_size = INTMAX_MAX;
+		r->reqbody_queue.upload_temp_file_size =
+		  (off_t)((1uLL << (sizeof(off_t)*8-1))-1);
 	}
 
 	{
@@ -981,13 +1040,19 @@ URIHANDLER_FUNC(cgi_is_handled) {
 		hctx->plugin_data = p;
 		hctx->cgi_handler = &ds->value;
 		memcpy(&hctx->conf, &p->conf, sizeof(plugin_config));
-		if (!light_btst(r->rqst_htags, HTTP_HEADER_UPGRADE))
+		if (__builtin_expect( (r->h2_connect_ext != 0), 0)) {
+		}
+		else if (!light_btst(r->rqst_htags, HTTP_HEADER_UPGRADE))
 			hctx->conf.upgrade = 0;
 		else if (!hctx->conf.upgrade || r->http_version != HTTP_VERSION_1_1) {
 			hctx->conf.upgrade = 0;
 			http_header_request_unset(r, HTTP_HEADER_UPGRADE,
 			                          CONST_STR_LEN("Upgrade"));
 		}
+		if (hctx->conf.upgrade) {
+			hctx->orig_reqbody_length = r->reqbody_length;
+			r->reqbody_length = -1;
+		}
 		hctx->opts.max_per_read =
 		  !(r->conf.stream_response_body /*(if not streaming response body)*/
 		    & (FDEVENT_STREAM_RESPONSE|FDEVENT_STREAM_RESPONSE_BUFMIN))
@@ -1064,29 +1129,25 @@ SUBREQUEST_FUNC(mod_cgi_handle_subrequest) {
 		if (chunkqueue_length(cq) > 65536 - 4096
 		    && (r->conf.stream_request_body & FDEVENT_STREAM_REQUEST_BUFMIN)){
 			r->conf.stream_request_body &= ~FDEVENT_STREAM_REQUEST_POLLIN;
-			if (-1 != hctx->fd) return HANDLER_WAIT_FOR_EVENT;
 		} else {
 			handler_t rc = r->con->reqbody_read(r);
-			if (!chunkqueue_is_empty(cq)) {
-				if (fdevent_fdnode_interest(hctx->fdntocgi) & FDEVENT_OUT) {
-					return (rc == HANDLER_GO_ON) ? HANDLER_WAIT_FOR_EVENT : rc;
-				}
-			}
-			if (rc != HANDLER_GO_ON) return rc;
-
-			/* CGI environment requires that Content-Length be set.
-			 * Send 411 Length Required if Content-Length missing.
-			 * (occurs here if client sends Transfer-Encoding: chunked
-			 *  and module is flagged to stream request body to backend) */
-			if (-1 == r->reqbody_length) {
-				return (r->conf.stream_request_body & FDEVENT_STREAM_REQUEST)
-				  ? http_response_reqbody_read_error(r, 411)
-				  : HANDLER_WAIT_FOR_EVENT;
-			}
+			if (rc != HANDLER_GO_ON
+			    && !(hctx->conf.upgrade && -1 == hctx->fd
+			         && rc == HANDLER_WAIT_FOR_EVENT))
+				return rc;
 		}
 	}
 
 	if (-1 == hctx->fd) {
+			/* CGI environment requires that Content-Length be set.
+			 * Send 411 Length Required if Content-Length missing.
+			 * (occurs here if client sends Transfer-Encoding: chunked
+			 *  and module is flagged to stream request body to backend) */
+			if (-1 == r->reqbody_length && !hctx->conf.upgrade) {
+				return (r->conf.stream_request_body & FDEVENT_STREAM_REQUEST)
+				  ? http_response_reqbody_read_error(r, 411)
+				  : HANDLER_WAIT_FOR_EVENT;
+			}
 		if (cgi_create_env(r, p, hctx, hctx->cgi_handler)) {
 			r->http_status = 500;
 			r->handler_module = NULL;
@@ -1094,6 +1155,8 @@ SUBREQUEST_FUNC(mod_cgi_handle_subrequest) {
 			return HANDLER_FINISHED;
 		}
 	} else if (!chunkqueue_is_empty(cq)) {
+		if (fdevent_fdnode_interest(hctx->fdntocgi) & FDEVENT_OUT)
+			return HANDLER_WAIT_FOR_EVENT;
 		if (0 != cgi_write_request(hctx, hctx->fdtocgi)) {
 			cgi_connection_close(hctx);
 			return HANDLER_ERROR;
