@@ -252,7 +252,14 @@ static void * mod_extforward_parse_forwarder(server *srv, const array *forwarder
         struct sock_addr_mask * const sm = fwd->addrs + fwd->addrs_used++;
         sm->bits = nm_bits;
         *nm_slash = '\0';
-        rc = sock_addr_from_str_numeric(&sm->addr, ds->key.ptr, srv->errh);
+        if (ds->key.ptr[0] == '['
+            && ds->key.ptr+1 < nm_slash && nm_slash[-1] == ']') {
+            nm_slash[-1] = '\0';
+            rc = sock_addr_from_str_numeric(&sm->addr,ds->key.ptr+1,srv->errh);
+            nm_slash[-1] = ']';
+        }
+        else
+            rc = sock_addr_from_str_numeric(&sm->addr,ds->key.ptr,  srv->errh);
         *nm_slash = '/';
         if (1 != rc) {
             free(fwd);
@@ -536,6 +543,7 @@ static int mod_extforward_set_addr(request_st * const r, plugin_data *p, const c
 	 * should not reuse same h2 connection for requests from different clients*/
 	if (hctx && !buffer_is_unset(&hctx->saved_remote_addr_buf)
 	    && r->http_version > HTTP_VERSION_1_1) { /*(e.g. HTTP_VERSION_2)*/
+		hctx->request_count = con->request_count;
 		if (extforward_check_proxy) /* save old address */
 			http_header_env_set(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_FOR"),
 			                    BUF_PTR_LEN(&hctx->saved_remote_addr_buf));
@@ -1063,14 +1071,17 @@ URIHANDLER_FUNC(mod_extforward_uri_handler) {
 
 	if (NULL == p->conf.headers) return HANDLER_GO_ON;
 
-	/* Do not reparse headers for same request, e.g after HANDER_COMEBACK
+	/* Do not reparse headers for same request, e.g. after HANDER_COMEBACK
 	 * from mod_rewrite, mod_magnet MAGNET_RESTART_REQUEST, mod_cgi
 	 * cgi.local-redir, or gw_backend reconnect.  This has the implication
 	 * that mod_magnet and mod_cgi with local-redir should not modify
-	 * Forwarded or related headers and expect effects here. */
+	 * Forwarded or related headers and expect effects here, though there
+	 * are some scenarios with HTTP/2 where headers are reparsed anyway,
+	 * due to the loose heuristic checking if request_count changed. */
 	handler_ctx *hctx = r->con->plugin_ctx[p->id];
-	if (NULL != hctx && !buffer_is_unset(&hctx->saved_remote_addr_buf)
-	    && hctx->request_count == r->con->request_count)
+	const int already =
+	  (NULL != hctx && !buffer_is_unset(&hctx->saved_remote_addr_buf));
+	if (already && hctx->request_count == r->con->request_count)
 		return HANDLER_GO_ON;
 
 	const buffer *forwarded = NULL;
@@ -1085,7 +1096,9 @@ URIHANDLER_FUNC(mod_extforward_uri_handler) {
 		}
 	}
 
-	if (forwarded && is_connection_trusted(r->con, p)) {
+	if (forwarded
+	    && ((already && r->http_version > HTTP_VERSION_1_1) /*(HTTP_VERSION_2)*/
+	        || is_connection_trusted(r->con, p))) {
 		return (is_forwarded_header)
 		  ? mod_extforward_Forwarded(r, p, forwarded)
 		  : mod_extforward_X_Forwarded_For(r, p, forwarded);
@@ -1561,10 +1574,12 @@ static int mod_extforward_hap_PROXY_v2 (connection * const con,
       case 0x31:  /* UNIX domain socket */
         {
             char *src_addr = (char *)hdr->v2.addr.unx.src_addr;
-            char *z = memchr(src_addr, '\0', UNIX_PATH_MAX);
+            char *z = memchr(src_addr, '\0', sizeof(hdr->v2.addr.unx.src_addr));
             if (NULL == z) return -1; /* invalid addr; too long */
-            len = (uint32_t)(z - src_addr + 1); /*(+1 for '\0')*/
-            sock_addr_assign(&con->dst_addr, AF_UNIX, 0, src_addr);
+            len = (uint32_t)(z - src_addr);
+            /*if (0 == len) return -1;*//* abstract socket not supported; err?*/
+            if (0 != sock_addr_assign(&con->dst_addr, AF_UNIX, 0, src_addr))
+                return -1; /* invalid addr; too long */
             buffer_copy_string_len(&con->dst_addr_buf, src_addr, len);
         }
        #if 0 /*(dst_addr should be identical to src_addr for AF_UNIX)*/
