@@ -7,12 +7,14 @@
 #include "rand.h"
 #include "chunk.h"
 #include "h2.h"             /* h2_send_1xx() */
+#include "http_range.h"     /* http_range_config_allow_http10() */
 #include "fdevent.h"
 #include "fdlog.h"
 #include "connections.h"
 #include "sock_addr.h"
 #include "stat_cache.h"
 #include "plugin.h"
+#include "plugins.h"
 #include "plugin_config.h"  /* config_plugin_value_tobool() */
 #include "network_write.h"  /* network_write_show_handlers() */
 #include "reqpool.h"        /* request_pool_init() request_pool_free() */
@@ -25,10 +27,12 @@
 #endif
 
 #define PACKAGE_DESC PACKAGE_NAME "/" PACKAGE_VERSION REPO_VERSION
-static const buffer default_server_tag = { CONST_STR_LEN(PACKAGE_DESC)+1, 0 };
+static const buffer default_server_tag =
+  { PACKAGE_DESC "\0server", sizeof(PACKAGE_DESC), 0 };
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include "sys-setjmp.h"
 #include "sys-time.h"
 
 #include <string.h>
@@ -318,11 +322,13 @@ static server *server_init(void) {
 	config_init(srv);
 
 	srv->request_env = plugins_call_handle_request_env;
+	srv->plugins_request_reset = plugins_call_handle_request_reset;
 
 	srv->loadavg[0] = 0.0;
 	srv->loadavg[1] = 0.0;
 	srv->loadavg[2] = 0.0;
 	srv->stdin_fd = -1;
+	srv->default_server_tag = &default_server_tag;
 
 	log_con_jqueue = (connection *)(uintptr_t)&log_con_jqueue;/*(sentinel)*/
 
@@ -781,8 +787,10 @@ static void server_sockets_restore (server *srv) { /* graceful_restart */
     memset(&graceful_sockets, 0, sizeof(server_socket_array));
     memcpy(&srv->srv_sockets_inherited, &inherited_sockets, sizeof(server_socket_array));
     memset(&inherited_sockets, 0, sizeof(server_socket_array));
-    for (uint32_t i = 0; i < srv->srv_sockets.used; ++i)
+    for (uint32_t i = 0; i < srv->srv_sockets.used; ++i) {
         srv->srv_sockets.ptr[i]->srv = srv;           /* update ptr */
+        srv->srv_sockets.ptr[i]->sidx= (unsigned short)~0u;
+    }
     for (uint32_t i = 0; i < srv->srv_sockets_inherited.used; ++i)
         srv->srv_sockets_inherited.ptr[i]->srv = srv; /* update ptr */
 }
@@ -1265,6 +1273,8 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	if (!config_feature_bool(srv, "server.h1-discard-backend-1xx", 0))
 		http_response_send_1xx_cb_set(connection_send_1xx, HTTP_VERSION_1_1);
 
+	http_range_config_allow_http10(config_feature_bool(srv, "http10.range", 0));
+
 	if (0 != config_set_defaults(srv)) {
 		log_error(srv->errh, __FILE__, __LINE__,
 		  "setting default values failed");
@@ -1404,7 +1414,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 
 			if (pwd->pw_uid == 0) {
 				log_error(srv->errh, __FILE__, __LINE__,
-				  "I will not set uid to 0\n");
+				  "I will not set uid to 0.  Perhaps you should comment out server.username in lighttpd.conf\n");
 				return -1;
 			}
 
@@ -1418,7 +1428,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		if (NULL != grp) {
 			if (grp->gr_gid == 0) {
 				log_error(srv->errh, __FILE__, __LINE__,
-				  "I will not set gid to 0\n");
+				  "I will not set gid to 0.  Perhaps you should comment out server.groupname in lighttpd.conf\n");
 				return -1;
 			}
 		}
@@ -1500,19 +1510,27 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 
 #ifdef HAVE_SIGACTION
 	memset(&act, 0, sizeof(act));
+	sigemptyset(&act.sa_mask);
+
 	act.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &act, NULL);
+
+#ifndef _MSC_VER
+	act.sa_flags = SA_NODEFER;
+	act.sa_handler = sys_setjmp_sigbus;
+	sigaction(SIGBUS, &act, NULL);
+	act.sa_flags = 0;
+#endif
+
 # if defined(SA_SIGINFO)
 	last_sighup_info.si_uid = 0,
 	last_sighup_info.si_pid = 0;
 	last_sigterm_info.si_uid = 0,
 	last_sigterm_info.si_pid = 0;
 	act.sa_sigaction = sigaction_handler;
-	sigemptyset(&act.sa_mask);
 	act.sa_flags = SA_SIGINFO;
 # else
 	act.sa_handler = signal_handler;
-	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
 # endif
 	sigaction(SIGINT,  &act, NULL);
@@ -1524,7 +1542,6 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	/* it should be safe to restart syscalls after SIGCHLD */
 	act.sa_flags |= SA_RESTART | SA_NOCLDSTOP;
 	sigaction(SIGCHLD, &act, NULL);
-
 #elif defined(HAVE_SIGNAL)
 	/* ignore the SIGPIPE from sendfile() */
 	signal(SIGPIPE, SIG_IGN);
@@ -1534,6 +1551,9 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	signal(SIGCHLD,  signal_handler);
 	signal(SIGINT,  signal_handler);
 	signal(SIGUSR1, signal_handler);
+#ifndef _MSC_VER
+	signal(SIGBUS,  sys_setjmp_sigbus);
+#endif
 #endif
 
 
@@ -1546,7 +1566,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		buffer * const tb = srv->tmp_buf;
 		buffer_clear(tb);
 		buffer_append_int(tb, srv->pid);
-		buffer_append_string_len(tb, CONST_STR_LEN("\n"));
+		buffer_append_char(tb, '\n');
 		if (-1 == write_all(pid_fd, BUF_PTR_LEN(tb))) {
 			log_perror(srv->errh, __FILE__, __LINE__, "Couldn't write pid file");
 			close(pid_fd);
